#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "yaftpd.h"
#include "client.h"
#include "server.h"
#include "log.h"

/*
 * add a client to double link list
 * and set default values
 */
CLIENT *
client_add(fd, head, server)
int fd;
CLIENT ** head;
SERVER * server;
{
  CLIENT * new_client, *client;
  int addrlen;

  LOG(L_FUNC, ("client_add() call:"));
  new_client = calloc(1, sizeof(CLIENT));
  if (new_client == (CLIENT *)NULL)
    return new_client;

/*
 * set def values
 * those are redundant because of calloc()
 * It's more for documentation
 *
  new_client->user = (char *)0;
  new_client->passwd = (char *)0;
  new_client->auth = 0;
  new_client->file.name = (char *)0;
  new_client->file.size = new_client->file.nwritten = 0;
  new_client->reply.msg = (char *)0;
  new_client->reply.size = new_client->reply.nwritten = 0;
*/

  addrlen = sizeof(new_client->addr);
  if (getpeername(fd, (struct sockaddr *)&new_client->addr, &addrlen) < 0)
  {
    LOG(L_LERR, ("  getpeername failed: %s", strerror(errno)));
  }
  new_client->uid = server->uid;
  new_client->gid = server->gid;
  new_client->server = server;
  new_client->root = (char *)strdup(server->root);
  new_client->cwd = (char *)strdup("/");
  new_client->timestamp = time((time_t *)0);
  new_client->sock_con = fd;
  new_client->sock_dat = -1;
  new_client->state = S_COMM;
  new_client->ostate = new_client->state;
  new_client->file.cursor = (caddr_t)-1;

  new_client->prev = new_client->next = (CLIENT *)0;

/* insert */
  if (*head == (CLIENT *)NULL)
  {
    *head = new_client;
    LOG(L_VERB, ("  Head is client %d", fd));
  }
  else
  {
    LOG(L_VERB, ("  inserting new client %d", fd));
    for (client = *head; client->next != (CLIENT *)NULL;
         client = client->next)
      ;
    client->next = new_client;
    new_client->prev = client;
  }

  for (client = *head; client != (CLIENT *)NULL; client = client->next)
    LOG(L_VERB, ("  list -> r: %d w: %d", client->sock_con, client->sock_dat));

  return new_client;
}
