#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include "yaftpd.h"
#include "client.h" /* for MUNMAP */
#include "log.h"

/* if the argument start with '-' we assume it's an option
 * for now not much is supported except -l
 */
int
ftp_list(arg, client)
char * arg;
CLIENT * client;
{
  char *opt = NULL;
  char *file = NULL;
  caddr_t buffer;
  off_t size;

  LOG(L_FUNC, ("ftp_list() call:"));
  LOG(L_COMM, ("  LIST/NLST <%s>", (arg == (char *)0) ? "(null)" : arg));

  if (client->auth <= 0)
  {
    LOG(L_DENY, (" not login, denied"));
    reply(client, "%d Not logged in", 530);
    return -1;
  }

/* data port must have been set first */
  if (client->sock_dat < 0 || client->state != S_DATA)
    return reply(client, "%d Need Pasv or Port first", 425);

/* options to LIST ? */
  if (arg != (char *)0 && arg[0] == '-') 
  {
    ++arg; /* mv pass the '-' */
    opt = strchr(arg, ' ');
    if (opt != (char *)0)                     /* case "-l file" */
    {
      *opt = '\0';
      if (++opt != (char *)NULL)
        file = opt;
    }
    else 
      opt = arg;
  } else if (arg != (char *)0 && *arg != 0)  /* no option just a filename */
    file = arg;

  if (file == (char *)0 || *file == 0)  /* no filename */
  {
    file = (char *)malloc(2);
    strcpy(file, ".");
  }
  else
  {
    file = path_filter((char *)0, file);
    if (file != (char *)0 && strcmp(file, "/") == 0)
      strcpy(file, ".");
  }


  LOG(L_VERB, ("  opt: %s file: %s", (opt == (char *)0) ? "(null)": opt, file));

  if (file == (char *)0)
    return reply(client, "%d local error, out of mem", 451);


/* we must be position in the correct dir for glob to work */
  if (chdir(client->root) < 0)
  {
    LOG(L_VERB, ("chdir %s failed: %s", client->root, strerror(errno)));
    free(file);
    return reply(client, "%d File access error: %s",
                 450, strerror(errno));
  }
  if (*file == '/')     /* absolute path */
    while (*file != 0 && *file == '/')
      file++;
  else                 /* relative path */
    if (strcmp(client->cwd, "/") != 0 && chdir(client->cwd + 1) < 0)
    {
      LOG(L_VERB, ("chdir %s failed: %s", client->cwd, strerror(errno)));
      free(file);
      return reply(client, "%d File access error: %s",
                   450, strerror(errno));
    }

  buffer = list(opt, file, &size); /* do the job */
  free(file);

  if (client->server->root != (char *)0)
    chdir(client->server->root);

  if (buffer == (caddr_t)-1)
    return reply(client, "%d No more space to mmap()", 451);
  
  if (client->file.name != (char *)0)   /* should not be */
  {
    LOG(L_VERB, ("  in ftp_list filename not free ????"));
    free(client->file.name);
  }

  client->file.name = (char *)0;
  if (client->file.cursor != (caddr_t)-1)
    MUNMAP(client);
  client->file.cursor = buffer;
  client->file.size = size;
  client->file.nwritten = 0;
  client->state = S_DATA_W;
  LOG(L_VERB, ("  ready to send listing <%d>", client->file.size));
  return reply (client, "%d File list ready, about to send ", 150);
} 

#ifdef DRIVER

#include <stdio.h>
#ifdef __STDC__
#include <stdarg.h>
#else
#include <varags.h>
#endif

main(argc, argv)
int argc;
char ** argv;
{
  CLIENT client;
  SERVER server;
  char buf[100] = "";
  char c[100] = "*";

  memset(&client, 0, sizeof(client));
  memset(&server, 0, sizeof(server));
  server.root = (char *)strdup("/");
  client.auth = 1;
  client.server = &server;
  client.root = (char *)strdup("/home/alainm/Yaftpd");
  client.cwd = (char *)strdup("/");
  client.sock_dat = 1;
  client.state = S_DATA;

  initlog("LIST");
  loglevel = 20;

  ftp_list(buf, &client);

  if (client.file.cursor == (caddr_t)-1)
   printf("error");
  else
  write(1, client.file.cursor, client.file.size); 

  printf("COUNT %d\n", client.file.size);
  
}
#endif
