#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <errno.h>
#include <sys/filio.h>
#include <unistd.h>

#include "yaftpd.h"
#include "client.h"
#include "log.h"

#ifdef DRIVER
int log();
#endif

/*
 * For PASV not to be a security risk
 * the client must connect first before
 * issuing the RETR/PUT
 */
int
ftp_pasv(arg, client)
char * arg;
CLIENT * client;
{
  struct sockaddr_in host;
  int flag = 1, len;
  char * a, * p;

  LOG(L_FUNC, ("fpt_pasv() call"));

  if (client->auth <= 0)
  {
    LOG(L_DENY, (" not login, denied"));
    reply(client, "%d Not logged in", 530);
    return -1;
  }

/* before opening a new socket close the old one if necessary */  
  client->sock_dat > 0 && close(client->sock_dat); 

  client->sock_dat = socket(AF_INET, SOCK_STREAM, 0);
  if (client->sock_dat < 0)
    return reply(client, "%d cannot create data socket", 425);

  len = sizeof(host);
  memset((char *) &host, 0, len);
  if (getsockname(client->sock_con, (struct sockaddr *)&host, &len) < 0)
  {
    close(client->sock_dat);
    client->sock_dat = -1;
    return reply(client, "%d cannot getsockname data ", 425);
  }

  host.sin_port = 0;
  if (bind(client->sock_dat, (struct sockaddr *)&host, len) < 0)
  {
    close(client->sock_dat);
    client->sock_dat = -1;
    return reply(client, "%d cannot bind data socket", 425);
  }
   
  listen(client->sock_dat, 1);

/* set the socket nonblocking */
  if (ioctl(client->sock_dat, FIONBIO, &flag))
  {
    close(client->sock_dat);
    client->sock_dat = -1;
    return reply(client, "%d cannot set nonblocking socket", 425);
  }

  if (getsockname(client->sock_dat, (struct sockaddr *)&host, &len) < 0)
  {
    close(client->sock_dat);
    client->sock_dat = -1;
    return reply(client, "%d cannot getsockname data ", 425);
  }


  client->state = S_PROG_ACC;

/* fill out the answer */
  a = (char *)&host.sin_addr;
  p = (char *)&host.sin_port;

#define UC(i) (((int)i) & 0xff)
  LOG(L_COMM, ("  PASV (%d,%d,%d,%d,%d,%d)",
               UC(a[0]), UC(a[1]), UC(a[2]), UC(a[3]),
               UC(p[0]), UC(p[1])));

  return reply(client, 
               "%d Entering Passive Mode (%d,%d,%d,%d,%d,%d)", 227,
               UC(a[0]), UC(a[1]), UC(a[2]), UC(a[3]),
               UC(p[0]), UC(p[1]));
}

#ifdef DRIVER

#include <stdio.h>
#ifdef __STDC__
#include <stdarg.h>
#else
#include <varags.h>
#endif

int log() {return 1;}

void
main()
{
  CLIENT client;

  client.state = S_COMM;
  client.auth = 1;
  client.sock_con = 1;
  client.sock_dat = -1;

  ftp_pasv((char *)NULL, &client);
}  
#endif
