#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <errno.h>
#include <sys/filio.h>
#include <unistd.h>
#include <stdlib.h>

#include "yaftpd.h"
#include "client.h"
#include "log.h"

/* tokenize the args
 * we shall no assume that IP is v4
 * so the last two is port number
 * and before is IP
 */
static int
parse_addr(buffer, host)
char * buffer;
struct sockaddr_in * host;
{
  static struct in_addr saddr;
  long port = -1;
  int got_p2 = 0;
  int n;

  memset((char *) host, 0, sizeof(struct sockaddr_in));  

/* get to the end of the string and take the last two components
 * they are assume to be the port number
 */
  for (n = strlen(buffer), n--; n > 0; n--)
  {
    if (buffer[n] != ',')
      continue;
    buffer[n] = '\0';
    if (!got_p2)
    {
      port = atoi(buffer + n + 1);
      got_p2 = 1;
    }
    else
    {
      port = atoi(buffer + n + 1)*256 + port;
      break;
    }
  } 
/* got a valid port ? */
  if (port < 0)
    return -1;

  for (n = 0; buffer[n] != '\0'; n++)
    if (buffer[n] == ',')
      buffer[n] = '.';

/* got a valid address ? */
  saddr.s_addr = inet_addr(buffer);
  if (saddr.s_addr == -1)
    return -1;

  host->sin_family = AF_INET;
  host->sin_addr.s_addr = saddr.s_addr;
  host->sin_port = htonl(port);
  return 1;
}

#ifndef DRIVER

/* we just set up the socket descriptor and don't do the actual
   connection, it would be a security risk if somene else
   guess the port and bind first we must wait for the
   RET or STOR command before sending the file
 */
int
ftp_port(arg, client)
char * arg;
CLIENT * client;
{
  struct sockaddr_in host;
  int flag = 1;

  LOG(L_FUNC, ("ftp_port() call"));
  LOG(L_COMM, ("  PORT %s", arg));

  if (client->auth <= 0)
  {
    LOG(L_DENY, (" not login, denied"));
    reply(client, "%d Not logged in", 530);
    return -1;
  }

  if (arg == (char *)0 || *arg == '\0')
    return reply(client, "%d Syntax error in parameter", 501);

  if (parse_addr(arg, &host) < 0)
    return reply(client, "%d PORT refused, parameter error", 504);

/* before opening a new socket close the old one if necessary */  
  client->sock_dat > 0 && close(client->sock_dat); 

  client->sock_dat = socket(AF_INET, SOCK_STREAM, 0);
  if (client->sock_dat < 0)
    return reply(client, "%d cannot create data socket", 425);

/* set the socket nonblocking */
  if (ioctl(client->sock_dat, FIONBIO, &flag))
  {
    close(client->sock_dat);
    client->sock_dat = -1;
    return reply(client, "%d cannot set nonblocking socket", 425);
  }

  client->state = S_PROG_CON;
  connect(client->sock_dat, (struct sockaddr *)&host, sizeof(host));

  return 1;
}
#endif

#ifdef DRIVER

main()
{
  char buffer[120];
  struct sockaddr_in host;

  sprintf(buffer, "%d,%d,%d,%d,%d,%d", 132,206,69,4,10,5);
  if (parse_addr(buffer, &host) < 0)
    printf("Erreur\n");
  else 
    printf("addr: %s  port %ld\n", inet_ntoa(host.sin_addr),
            ntohl(host.sin_port));
} 
#endif
