#include <string.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include "yaftpd.h"
#include "client.h"
#include "log.h"

int
ftp_retr(arg, client)
char * arg;
CLIENT * client;
{
  struct stat st;
  char * file;
  char * filename;
  int fd;

  LOG(L_FUNC, ("ftp_retr() call:"));

  if (client->auth <= 0)
  {
    LOG(L_DENY, (" not login, denied"));
    reply(client, "%d Not logged in", 530);
    return -1;
  }

/* data port must have been set first */
  if (client->sock_dat < 0 || client->state != S_DATA)
    return reply(client, "%d Need Pasv or Port first", 425);

  if (arg == (char *)0 || *arg == 0)
    return reply(client, "%d Syntax error in parameter", 501);

/* filter path */
  if (arg[0] == '/')
    file = path_filter((char *)0, arg);
  else
    file = path_filter(client->cwd, arg);

  if (file == (char *)0)
    return reply(client, "%d local error, out of mem", 451);

/* we have to act from within client->root */
  filename = (char *)malloc(strlen(file) + strlen(client->root) + 1);
  if (filename == (char *)0)
  {
    free(file);
    return reply(client, "%d local error, out of mem", 451);
  }

  sprintf(filename, "%s%s", client->root, file);

  fd = open(filename, O_RDONLY);
  if (fd == -1)
  {
    free(file);
    free(filename);
    return reply(client, "%d Can't read: %s: %s", 451, file, strerror(errno));
  }

  LOG(L_PATH, ("  filter %s -> %s", 
               (file == (char *)0) ? "(null)" : file,
               (filename == (char *)0) ? "(null)" : filename));
  free(filename);
  if (fstat(fd, &st) < 0)
  {
    free(file);
    close(fd);
    return reply(client, "%d Can't stat file %s", 550, file);
  }
  if (! S_ISREG(st.st_mode))
  {
    free(file);
    close(fd);
    return reply(client, "%d Not a plain file %s", 550, file);
  }
 
  if (client->file.name != (char *)0)
    free(client->file.name);
  client->file.name = file;

/* mmap() the file to memory and mark state as ready to send */
  client->file.cursor = mmap(0, st.st_size,
                             PROT_READ, MAP_SHARED, fd, 0);

  close(fd);
  if (client->file.cursor == (caddr_t)-1)
  {
    free(client->file.name);
    client->file.name = (char *)0;
    return reply(client, "%d local error, mmap %s", 451, strerror(errno));
  }


  LOG(L_COMM, ("  RETR %s", (file == (char *)0) ? "(null)" : file));

  client->state = S_DATA_W;
  client->file.size = st.st_size;
  client->file.nwritten = 0;
  return reply (client, "%d File %s ready, about to send", 150,
                         (file== (char *)0) ? "(null)" : file);
} 
