#include <sys/types.h> /* size_t and misc */
#include <dirent.h> /* opendir() readdir() */
#include <sys/stat.h> /* opendir() readdir() */
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>

#ifdef DRIVER
#include <stdio.h>
#endif

#include "glob/glob.h"
#include "yaftpd.h"
#include "log.h"
#include "client.h"

#if 0
#ifndef HAVE_SNPRINTF
extern int snprintf __P((char *, size_t, const char *, ...));
#endif
#endif

/*          free(buffer); */
#define CHECK_BUFFER(need, size, buffer, gl) \
      if(need >= *size) \
      { \
        caddr_t tmp; \
        tmp = give_chunk(buffer, size); \
        if (tmp == NULL) \
        { \
          globfree(&gl); \
          return (caddr_t)-1; \
        } \
        buffer = tmp; \
      } 

/* give_chunk map() a small chunk of memory
 * + size and return the address
 */
static caddr_t
give_chunk(buffer, size)
caddr_t buffer;
off_t * size;
{
  static off_t page;
  int fd;

  LOG(L_FUNC, ("give_chunk() call"));

  if (page == 0)
    page = (off_t)getpagesize();

  *size += page;

  fd = open("/dev/zero", O_RDWR);
  if (fd == -1)
  {
    if (buffer != (caddr_t)-1) 
      munmap(buffer, *size - page);
    *size = 0;
    return (char *)0;
  }
  if (buffer == (caddr_t)-1)
    buffer =  mmap(0, *size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
  else 
  { 
    caddr_t result;
    result = mmap(0, *size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
    memcpy(result, buffer, *size - page);
    munmap(buffer, *size - page);
    buffer = result;
  }

  if (buffer == (caddr_t)-1)
    *size = 0;

  LOG(L_VERB, ("  allocate %d", *size));
  close(fd);
  return buffer;
}

    
/* fill buffer */
char *
list(opt, file, size)
char * opt;
char * file;
off_t * size;
{
  glob_t gl;
  caddr_t buffer;
  char * dirname;
  char ** dirlist;
  int j; int count ;
#define MAXLINE 1024
  char line[MAXLINE];
 
  LOG(L_FUNC, ("list() call:")); 

/* glob file */
  memset(&gl, 0, sizeof(gl));
  switch (glob(file, GLOB_NOCHECK, 0, &gl))
  {
    case GLOB_ABEND:
    case GLOB_NOMATCH:
      return (char *)NULL;
    case GLOB_NOSPACE:
      return (char *)NULL;
    case 0:
    break;
    default:
      return (caddr_t)-1;
  }

/* allocate a chunk */
  *size = 0;
  buffer = give_chunk((caddr_t)-1, size);
  if (buffer == (caddr_t)-1)
  {
    globfree(&gl);
    return (caddr_t)-1;
  }

/* first time we show only files, and substitute them by null char */
  count = 0;
  dirlist = gl.gl_pathv;
  while((dirname = *dirlist++) != (char *)NULL)
  {
    struct stat st;
    if ((stat(dirname, &st) == 0) && !(S_ISDIR(st.st_mode)))
    {
      j = format(opt, dirname, line, sizeof(line));
      CHECK_BUFFER(count + j, size, buffer, gl);
      memcpy((buffer + count), line, j);
      count += j;
      *dirname = '\0';
    }
  }

/* second time we show the directories also */
  dirlist = gl.gl_pathv;
  while((dirname = *dirlist++) != (char *)NULL)
  {
    glob_t gl2;
    char * dirname2;
    char ** dirlist2;

    if (*dirname == '\0') continue;
    j = snprintf(line, sizeof(line), "\n%s:\n", dirname);
    CHECK_BUFFER(count + j, size, buffer, gl);
    memcpy((buffer + count), line, j);
    count += j;

    if (chdir(dirname) < 0) continue;
    memset(&gl2, 0, sizeof(gl2));
    switch (glob("*", GLOB_NOCHECK, 0, &gl2))
    {
      case GLOB_ABEND:
      case GLOB_NOMATCH:
        return (caddr_t)-1;
      case GLOB_NOSPACE:
        globfree(&gl);
        free(buffer); *size =0;
        return (caddr_t)-1;
      case 0:
        break;
      default:
        globfree(&gl);
        free(buffer); *size =0;
        return (caddr_t)-1;
    }

    dirlist2 = gl2.gl_pathv;
    while((dirname2 = *dirlist2++) != (char *)NULL)
    {
      j = format(opt, dirname2, line, sizeof(line));
      CHECK_BUFFER(count + j, size, buffer, gl);
      memcpy((buffer + count), line, j);
      count += j;
    }
    globfree(&gl2);
    chdir("..");
  }

  globfree(&gl);
  *size = count;
  return buffer;
}

#ifdef DRIVER


void main()
{
  char * buffer;
  off_t size;

  initlog("LIST");
  loglevel = 16;
  buffer = list("", "*", &size);

  if (buffer == NULL)
   printf("error\n");
  else
  write(1, buffer, size); 

  printf("COUNT %ld\n", (long)size);
  
}
#endif
