/*
 * To test: cc -DDRIVER log.c -o log
 * To use: cc -c log.c
 *        and link log.o with your libs
 */
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h> /* for struct stat and S_ISLNK */
#include <sys/stat.h> /* for struct stat and S_ISLNK */
#if __STDC__
#include <string.h> 
#else
#include <strings.h>
#endif
#include <stdio.h>
#ifdef __STDC__
#include <stdarg.h>
#else 
#include <varargs.h>
#endif

#include "log.h"

#ifndef HAVE_SNPRINTF
extern int snprintf __P((char *, size_t, const char *, ...));
extern int vsnprintf __P((char *, size_t, const char *, va_list));
#endif

#ifndef LOGPERM
#  define LOGPERM  (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP)
#endif

int loglevel;     /* set if no flags file */

static int logfile;  /* file dexciptor of the log file */

int
#ifdef __STDC__
initlog(char * file)
#else
initlog(file)
char * file;
#endif
{
  int fd;
  struct stat buf; /* check for symbolic links */
 

  logfile = fd = open(file, O_EXCL|O_CREAT|O_APPEND|O_WRONLY, LOGPERM);
  if (fd > 0) return fd;

/* be cautious about root */
  if (geteuid() == 0)
    if (lstat(file, &buf) >= 0)
      if (S_ISLNK(buf.st_mode)) return -1;

/* for normal dudes */
  return (logfile = open(file, O_CREAT|O_APPEND|O_WRONLY, LOGPERM));
}

#define BUFSIZE 1024

static char buffer[BUFSIZE];

int
#ifdef __STDC__
log(char * fmt, ...)
#else
log(va_alist)
va_dcl
#endif
{
#ifndef __STDC__
  char *fmt;    /* print format */
#endif
  va_list args; /* argument list */
  int size;

/* initialize argument list */
#ifdef __STDC__
  va_start(args, fmt);
#else
  va_start(args);
  fmt = va_arg(args, char *);  /* get debug level */
#endif

/* display remaining args */ 
  size = vsnprintf(buffer, sizeof(buffer) - 2, fmt, args);
/*  size = vsprintf(buffer, fmt, args);  display remaining args */ 
  va_end(args);  /* cleanup argument list */

  strcat(buffer, "\n");
  return write(logfile, buffer, size + 1);
}

#ifdef DRIVER
void
main()
{
  char broil = 'M';
  short stuff = 24680;
  int rude = 13579;
  float age = 6.276;
  double talk = 3.45e-12;
  long fellow = 15678L;
  static char ge[10] = "low";
  char *coal  = "black";

  initlog("LOG");
loglevel = 7;
  LOG(1, ("broil = %c", broil));
  LOG(2, ("stuff = %d", stuff));
  LOG(3, ("rude = %d", rude));
  LOG(4, ("age = %.3f", age));
  LOG(5, ("talk = %g", talk));
  LOG(6, ("fellow = %ld", fellow));
  LOG(7, ("ge = %s", ge));
  LOG(8, ("coal's address = %x coal = %s", coal, coal));
  LOG(9, ("broil = %c rude = %d age = %.3f ge = %s coal = %s",
       broil, rude, age, ge,coal));
}
#endif
