#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/*
 * the approach: disassemble the string with strtok()
 * reassemble everything with some parsing on ".." and "."
 * 
 * A double link list, and dome overhead
 */

#define NODE struct node 

NODE
{
  char * token;
  NODE * next;
  NODE * prev;
};

#define INIT(head, tail) tail = head = (NODE *)0

#define ADD(head, tail, string) \
{ \
  NODE * p; \
  p = (NODE *)malloc(sizeof(NODE)); \
  if (p != (NODE *)0) \
  { \
    p->token = string; \
    if (tail == (NODE *)0) \
    { \
      head = tail = p; \
      p->next = (NODE *)0; \
      p->prev = (NODE *)0; \
    } \
    else \
    { \
      tail->next = p; \
      p->prev = tail; \
      p->next = (NODE *)0; \
      tail = p; \
    } \
  } \
}

#define DELETE(head, tail, p) \
  if (p != (NODE *)0) \
  { \
    if (p->prev != (NODE *)0) \
      p->prev->next = p->next; \
    else \
      head = p->next; \
    if (p->next != (NODE *)0) \
      p->next->prev = p->prev; \
    else \
      tail = p->prev; \
    free(p); \
    p = (NODE *)0; \
  }

#define CLOSE(head) \
{ \
   NODE * p,  * pnext; \
   for (p = head; p != (NODE *)0; p = pnext) \
   { \
     pnext = p->next; \
     free(p); \
   } \
   head = (NODE *)0; \
}

char *
path_filter(oldpath, newpath)
char * oldpath;
char * newpath;
{
  char * tokptr, * strptr;
  int i;
  NODE * head;
  NODE * tail;
  NODE * p;
  NODE * pnext, * pprev;

/* init list */
  INIT(head, tail);

/* insert the old path on the list */
  strptr = oldpath;
  if (strptr != (char *)0)
    while ((tokptr = strtok(strptr, "/")) != (char *)0)
    {
      ADD(head, tail, tokptr);
      strptr = (char *)0;
    }

/* insert the new path on the list */
  strptr = newpath;
  if (strptr != (char *)0)
    while ((tokptr = strtok(strptr, "/")) != (char *)0)
    {
      ADD(head, tail, tokptr);
      strptr = (char *)0;
    }
 
/* do some parsing on the list to get rid of "." and ".."*/
  for (p = head; p != (NODE *)0; p = pnext)
  {
    pnext = p->next;
    pprev = p->prev;
    if (strcmp(p->token, ".") == 0)
    {
      DELETE(head, tail, p);
      continue;
    }
    if (strcmp(p->token, "..") == 0)
    {
      DELETE(head, tail, p);
      DELETE(head, tail, pprev);
    }
  }

/* some OS can't take realloc argument NULL */
  strptr = (char *)malloc(2);
  if (strptr == (char *)0)
  {
    CLOSE(head);
    return (char *)0;
  }

  if (head == (NODE *)0)
    strcpy(strptr, "/"); 
  else
  {
    strcpy(strptr, ""); 
    for (p = head, i = 0; p != (NODE *)0; p = p->next)
    {
      i += strlen(p->token);
      i += 3; /* because: (2) -> '/\0' (1) -> '\0' */ 
      strptr = (char *)realloc(strptr, i);
      if (strptr == (char *)0)
      {
        CLOSE(head);
        free(strptr);
        return (char *)0;
      }
      strcat(strptr, "/");
      strcat(strptr, p->token);
    }
  }
 
  CLOSE(head);
  return strptr;
}

#ifdef DRIVER
void
main()
{
  /*static char old[30] = "/";*/
  char new[1024] = "/home/alainm/yaftpd-0.1/pub/mirrors/ftp.netscape.com/pub/communicator/4.0/4.0b3/windows/n32e40b3.exe";


  printf(" --> %s\n", path_filter((char *)0, new));
} 
#endif
