#include <stdlib.h> /* atol */
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> /* defines the open flags */
#include "yaftpd.h" /* need typdef SERVER */
#include "cmd.h" /* need typdef COMMAND */

#define MAXLINE 80
#define DIRECTIVE COMMAND
#define find_directive find_command
static int dir_uid(), dir_gid();
static int dir_port();
static int dir_root();
static int dir_loglevel(), dir_logfile();
static int dir_limit(), dir_redirect(), dir_banner();

/* the third field(help) is more for doumentation purposes
 * it is not access and should be (char *)NULL reallly
 */
static DIRECTIVE directives[] = {
	{ "uid", dir_uid, "uid number" },
	{ "gid", dir_gid, "gid number" },
	{ "port", dir_port, "port number" },
	{ "root", dir_root, "root path-name" },
	{ "loglevel", dir_loglevel, "loglevel number" },
	{ "logfile", dir_logfile, "logfile file-name" },
	{ "limit",  dir_limit,  "limit addr number" },
	{ "redirect", dir_redirect, "redirect addr" },
	{ "banner", dir_banner, "banner file-name" },
  { (char *)NULL, (int (*)())NULL, (char *)NULL }
};

extern int loglevel;
extern char * logfile;
extern long port;

int
server_cfg (filename, server)
char *filename;
SERVER * server;
{
  FILE * fp;
  char line[MAXLINE]; 
  char *word;
  int i;
  DIRECTIVE * directive;

  fp = fopen(filename, "r");
  if (fp == NULL)
    return -1;

  while(fgets(line, MAXLINE, fp) != NULL)
  {
/* get rid of comments */
    word = strchr(line, '#');
    if (word != NULL)
      *word = '\0';

/* chop NEWLINE */
    word = strchr(line, '\n');
    if (word != NULL)
      *word = '\0';

/* empty line ? */
    if (line[0] == '\0')
      continue;

/* isolate the directive */
    i = 0;
    while (line[i] && isspace(line[i]))
      i++;
    word = line + i;
    while (line[i] && !isspace(line[i]))
      i++;
    if (line[i])
      line[i++] = '\0';

/* is it a valid directive ? */
    directive = find_directive (word, directives);
    if (directive == NULL)
      continue;

/* Get argument to directive, if any. */
    while (line[i] && isspace(line[i]))
      i++;
    word = line + i;
    stripwhite(word);

/* Call the function. */
    i = (*(directive->dispatch)) (word, server);
    if (i < 0)
    {
      fprintf(stderr,"%s: %s\n", directive->doc, strerror(errno));
      fclose(fp);
      return i;
    }
  }

  fclose(fp);
  return 0;
}

static int
dir_uid(arg, server)
char * arg;
SERVER *server;
{
  return ((server->uid = (uid_t)atol(arg)) == 0) ? -1 : 1;
} 

static int
dir_gid(arg, server)
char * arg;
SERVER *server;
{
  return ((server->gid = (gid_t)atol(arg)) == 0) ? -1 : 1;
} 

static int
dir_port(arg, server)
char * arg;
SERVER *server;
{
  return ((port = atol(arg)) == 0) ? -1 : 1;
} 

static int
dir_loglevel(arg, server)
char * arg;
SERVER *server;
{
  return loglevel = atoi(arg);
} 

static int
dir_root(arg, server)
char * arg;
SERVER *server;
{
  char * buffer;
	buffer = (char *)strdup(arg);
  if (buffer == (char *)NULL)
  {
    fprintf(stderr,"dir_root,out of mem: %s\n", strerror(errno));
    return -1;
  }
  if (buffer[0] != '/')
  { 
    fprintf(stderr,"must be abolute path\n");
    return -1;
  }
  if (chdir(buffer) < 0)
  {
    fprintf(stderr, "can't change dir to %s", buffer);
    return -1;
  }
  server->root = buffer;
  return 1;
} 

static int
dir_logfile(arg, server)
char * arg;
SERVER *server;
{
  if (arg == (char *)0 || *arg == '\0')
    return -1;

  return (logfile = (char *)strdup(arg)) == (char *)0 ? -1 : 1;
} 

static int
dir_redirect(arg, server)
char * arg;
SERVER *server;
{
  return ((server->redirect = (char *)strdup(arg)) == NULL) ? -1 : 1;
} 

static int
dir_banner(arg, server)
char * arg;
SERVER *server;
{
  return (server->banner = open(arg, O_RDONLY));
} 

static int
dir_limit(line, server)
char * line;
SERVER *server;
{
  int i;
  char * arg;
  LIMIT * limit;
  LIMIT * plimit;

  if (line == (char *)NULL) return -1;

  plimit = (LIMIT *)calloc(1, sizeof(LIMIT));
  if (plimit == (LIMIT *)NULL) return -1;
  
/* Isolate arguments */
  i = 0;
  while (line[i] && isspace(line[i])) i++;
  arg = line + i;
  while (line[i] && !isspace(line[i])) i++;
  if (line[i]) line[i++] = '\0';

/* set the variables */
  plimit->addr= (char *)strdup(arg);
  if (plimit->addr == (char *)NULL) return -1;
  plimit->count = atoi(arg + i);

  if (server->limit == NULL)
  {
    server->limit = plimit;
    return 1;
  }
  
  for (limit = server->limit->next; limit != NULL; limit = limit->next);

  limit = plimit;

  return 1;
} 

#ifdef DRIVER

#include <stdio.h>

int loglevel;
int logfile;
SERVER server;
long port = 1234;
void 
main()
{
  int i;
  LIMIT *limit;

  i = server_cfg("yaftpd.conf", &server);
  if (i < 0) printf("error in config file\n");
  printf("uid = <%d>\n", (int)server.uid);
  printf("gid = <%d>\n", (int)server.gid);
  printf("port = <%d>\n", (int)port);
  printf("root = <%s>\n", (server.root) ? server.root: "");
  printf("redirect = <%s>\n", (server.redirect) ? server.redirect : "");
  printf("logfile = <%s>\n", logfile);
  printf("banner = <%d>\n", server.banner);
  printf("loglevel = <%d>\n", loglevel);
  for (limit = (&server)->limit; limit; limit = limit->next)
    printf("limit = %s %d\n", limit->addr, limit->count);
}
#endif
