#include <sys/types.h>
#include <fcntl.h>  /* fcntl() */
#include <sys/socket.h> /* for accept/getsockopt */
#include <errno.h>

#include "yaftpd.h"
#include "server.h"
#include "client.h"
#include "log.h"

#ifdef DRIVER

#include <stdio.h>
#include <signal.h>

int test_accept(int);
#define ftp_quit(p) 1
#define parsecmd(buf, client) LOG(6, ("Command: %s", buf))
#define accept(a,b,c) 1

#endif

static CLIENT * Head; /* head of the list */

int
server_loop(server)
SERVER * server;
{
  fd_set  rset, wset, eset;
  fd_set  t_rset, t_wset, t_eset;
  int maxfd;
  int result;
  int nread;
  CLIENT *client ;
#define MAXLINE 120
  char buf[MAXLINE];

  LOG(L_FUNC, ("server_loop() call:"));
  FD_ZERO(&rset);
  FD_ZERO(&wset);
  FD_ZERO(&eset);

/* set sock  for server */
  LOG(L_SELE, (" Listen to Head descriptor %d", server->sockfd));
  client_add(server->sockfd, &Head, server);
  FD_SET(server->sockfd, &rset);
  FD_SET(server->sockfd, &eset);
  maxfd = server->sockfd;

  for ( ; ; ) 
  {
    t_rset = rset; t_wset = wset; t_eset = eset;
    result = select(maxfd + 1, &t_rset, &t_wset, &t_eset, NULL);

/* ??? bad filedes ?? delete them */      
    if (result < 0)
    {
      LOG(L_SELE, ("  we got a bad fd in select: %s", strerror(errno)));
      switch(errno)
      {
         case EBADF:
           for (client = Head->next; client != (CLIENT *)NULL;
                client = client->next)
           {
             if (fcntl(client->sock_con, F_GETFL, 0) < 0)
             {
               LOG(L_SELE, ("  FD_CLR(%d) con bad fd", client->sock_con));
               FD_CLR(client->sock_con, &rset);
               FD_CLR(client->sock_con, &eset);
               FD_CLR(client->sock_con, &wset);
               client_del(client);
             }
             if (client->sock_dat > 0 && 
                 fcntl(client->sock_dat, F_GETFL, 0) < 0)
             {
               LOG(L_SELE, ("  FD_CLR(%d) dat bad fd", client->sock_dat));
               FD_CLR(client->sock_dat, &rset);
               FD_CLR(client->sock_dat, &wset);
               client->sock_dat = -1;
             }
           }
         case EINTR:
         case EINVAL:
         default:
         break;
      }
      continue;
    }

/* accept new client request */
    if (FD_ISSET(server->sockfd, &t_rset))
    {
      int clifd;
      CLIENT * new_client;

      LOG(L_SELE, ("********** new client arriving **********"));
#ifdef DRIVER
      clifd = test_accept(server->sockfd);
#else
      clifd = server_accept(server->sockfd);
#endif
      if (clifd < 0)
        LOG(L_DENY, ("  accept client refuse: %d", clifd));
      else
        if ((new_client = client_add(clifd, &Head, server)) != (CLIENT *)0)
        {
#ifndef DRIVER
          reply(new_client, "%d Yaftpd ready", 220);
#endif
          LOG(L_SELE, ("  FD_SET(%d)", clifd));
          FD_SET(clifd, &rset);
          if (clifd > maxfd) maxfd = clifd;  /* max fd for select() */
        }
    }

/* check clients to see who's calling */
    for (client = Head->next; client != (CLIENT *)NULL;
         client = client->next)
    {
/* COMMAND state someone wants to talk */
      if (FD_ISSET(client->sock_con, &t_eset) ||
          FD_ISSET(client->sock_con, &t_rset))
      {
/******* before doing anything set uid/gid **********/
        server->uid == (uid_t)0 && seteuid(client->uid);
        server->gid == (gid_t)0 && setegid(client->gid);

        LOG(L_SELE, ("******* request From CTRL %d ************",client->sock_con)); 

        nread = readline(client->sock_con, buf, MAXLINE);

        if (nread <= 0) /* client has closed conn */
        {
          LOG(L_SELE, ("  closed: %d", client->sock_con));
          FD_CLR(client->sock_con, &rset);
          FD_CLR(client->sock_con, &eset);
          switch(client->state)
          {
            case S_PROG_CON:
            case S_DATA_W:
              (client->sock_dat >= 0) && FD_CLR(client->sock_dat, &wset);
            break;
            case S_PROG_ACC:
            case S_DATA_R:
              (client->sock_dat >= 0) && FD_CLR(client->sock_dat, &rset);
            break;
            case S_COMM_W:
              (client->sock_con >= 0) && FD_CLR(client->sock_con, &wset);
            break;
          }
          client_del(client); 
        }
        else
        {
          parsecmd(buf, client);
          switch(client->state)
          {
            case S_COMM_W:
              LOG(L_SELE, ("  FD_SET(%d) COMM_W for %d", client->sock_con,
                  client->sock_con));
              FD_SET(client->sock_con, &wset);
              if (client->sock_con > maxfd) maxfd = client->sock_con;
            break;
            case S_PROG_CON:
              LOG(L_SELE, ("  FD_SET(%d) PROG_CON for %d", client->sock_dat,
                  client->sock_con));
              FD_SET(client->sock_dat, &wset);
              if (client->sock_dat > maxfd) maxfd = client->sock_dat;
            break;
            case S_PROG_ACC:
              LOG(L_SELE, ("  FD_SET(%d) PROG_ACC for %d", client->sock_dat,
                  client->sock_con));
              FD_SET(client->sock_dat, &rset);
              if (client->sock_dat > maxfd) maxfd = client->sock_dat;
           break;
            case S_DATA_R: /* not implemented */
              if (client->sock_dat > 0) /* if I need to send something */
              {
                LOG(L_SELE, ("  FD_SET(%d) read for %d", client->sock_dat,
                        client->sock_con));
                FD_SET(client->sock_dat, &rset);
                if (client->sock_dat > maxfd) maxfd = client->sock_dat;
              }
            break;
            case S_DATA_W:
              if (client->sock_dat > 0) /* if I need to send something */
              {
                LOG(L_SELE, ("  FD_SET(%d) send for %d", client->sock_dat,
                        client->sock_con));
                FD_SET(client->sock_dat, &wset);
                if (client->sock_dat > maxfd) maxfd = client->sock_dat;
              }
            break;
            case S_FIN:
              FD_CLR(client->sock_con, &rset);
              FD_CLR(client->sock_con, &eset);
              client_del(client); 
            break;
          }
        }
        server->uid == (uid_t)0 && seteuid(server->uid);
        server->gid == (gid_t)0 && setegid(server->gid);
        continue;
      } /* if COMMAND */

/* DATA state */
      if ((client->sock_dat > 0 && FD_ISSET(client->sock_dat, &wset)) ||
          (client->sock_dat > 0 && FD_ISSET(client->sock_dat, &rset)) ||
          FD_ISSET(client->sock_con, &wset))
      {
        server->uid == (uid_t)0 && seteuid(client->uid);
        server->gid == (gid_t)0 && setegid(client->gid);
        LOG(L_SELE, ("******* request From DATA %d ************",client->sock_dat)); 
        switch(client->state)
        {
          case S_PROG_ACC:
            {
              int fd;
              LOG(L_SELE, ("  FD_CLR(%d) PROG_ACC for %d", client->sock_dat,
                  client->sock_con));
              FD_CLR(client->sock_dat, &rset);
              fd = accept(client->sock_dat, (struct sockaddr *)0, (int *)0); 
              close(client->sock_dat);
              client->sock_dat = fd;
              if (fd > 0)
                client->state = S_DATA;
              else
              {
                reply(client, "%d Can't open data conn", 425);
                client->state = S_COMM;
              }
            }
          break;
          case S_PROG_CON:
            {
              int len, err;
              LOG(L_SELE, ("  FD_CLR(%d) PROG_CON for %d", client->sock_dat,
                  client->sock_con));
              FD_CLR(client->sock_dat, &wset);
#ifndef DRIVER
/* on my system(Solaris) getsockopt fail if connect failed ???
 * we'll test for the value of  err
 * if err is not 0 then something went wrong close data
 */
              err = 0;
              len = sizeof(err);
              getsockopt(client->sock_dat, SOL_SOCKET,
                             SO_ERROR, (char *)&err, &len);
              if (err == 0)
              {
                reply(client, "%d PORT/PASV command ok", 200);
                client->state = S_DATA;
              }
              else
              { 
                reply(client, "%d  can't connect to port", 425);
                client->sock_dat = -1;
                client->state = S_COMM;
              }
#endif
            }
          break;
          case S_DATA_R: /* not implememted */
          break;
          case S_DATA_W:
            if (client_write(client) < 0)
            {
              LOG(L_SELE, (" FD_CLR(%d) client %d finish", client->sock_dat,
                  client->sock_con));
              FD_CLR(client->sock_dat, &wset);
              close(client->sock_dat);
              client->sock_dat = -1;
            }
          break;
          case S_COMM_W:
            if (client_reply(client) < 0)
            {
              LOG(L_SELE, (" FD_CLR(%d) comm send finish", client->sock_con));
              FD_CLR(client->sock_con, &wset);
            }
          break;
        }
        server->uid == (uid_t)0 && seteuid(server->uid);
        server->gid == (gid_t)0 && setegid(server->gid);
      } /* if DATA */

    } /* for check */

  } /* for server_loop */

/*NOTREACHED*/
  LOG(1, ("HELP ! HELP ! HELP ! I'M DYING HAAAAA"));
  return -1;
}

#ifdef DRIVER


int
readline(fd, buf, n)
int fd;
char *buf;
int n;
{
  int i;
  unsigned char c;

  LOG(L_FUNC, ("readline() call:"));
  for (i = 0; i < n -1 && read(fd, &c, 1); i++)
  {
    if (c == '\n') break;
    buf[i] = c;
  }
  buf[i] = '\0';
  LOG(L_VERB, ("  Read: %s", buf));
  return i;
}

int 
test_accept(fd)
int fd;
{
   char buf[20];
   int cl;

   LOG(L_FUNC, ("test_accept() call:"));
   readline(fd, buf, 20);
   cl = atoi(buf);
   LOG(L_VERB, ("  accepting : %d %s", cl, buf));

   return ( cl > 0) ? cl : -1;
}

int
send_fd(whom, fd)
int whom;
int fd;
{
  char buf[10];
  int n;
  
  n = sprintf(buf, "%d\n", fd);
  printf("sending to %d %d\n", whom, fd);
  return write(whom, buf, n);
}

void
main()
{
  int rd[2], c1[2], c2[2];
  time_t date;
  pid_t child;
  SERVER server;

  if (pipe(rd) < 0 || pipe(c1) < 0 || pipe(c2) < 0)
  { 
    perror("error pipe");
    exit(-1);
  }

  initlog("LOOP");
  loglevel = 20;
  date = time(NULL);
  LOG(1, ("\nTIME Start: %s", ctime(&date)));

  switch (child = fork())
  {
    case 0:
      LOG(6, ("in child\n"));
      server.sockfd = rd[0];
      server_loop(&server);
    break;
    case -1:
      perror("fork"); 
    break;
    default:
      sleep(3); /* allow child to settle down */
      printf("in parent\n");

      send_fd(rd[1], c1[0]);
      write(c1[1], "Hello c1\n", strlen("Hello c1\n"));

      send_fd(rd[1], c2[0]);
      write(c2[1], "Hello c2\n", strlen("Hello c2\n"));

      write(c1[1], "Hello dear, c1\n", strlen("Hello dear, c1\n"));
      write(c2[1], "Hello dear, c2\n", strlen("Hello dear, c2\n"));
      close(c1[1]);
    break;
  }

sleep(3); /* give child tome to brieve */
  kill(child,SIGKILL);
  
}
#endif
