/*
 yaftpd: a non-forking ftpd
 version 0.0
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _YAFTPD_H
#define _YAFTPD_H

#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <netinet/in.h>
#ifdef __STDC__ 
#include <stdarg.h> 
#else 
#include <varargs.h>
#endif

#ifndef __P
#  ifdef __STDC__
#    define __P(p) p
#  else
#    define __P(p) ()
#  endif
#endif

/* simple list that holds the limit of connection per class/addr ?? */
/* not implemented */
typedef struct LIMIT
{
  char * addr;
  int count;
  struct LIMIT * next; 
} LIMIT;

/* A structure which contains information on the server */
typedef struct SERVER
{
  uid_t uid;			     /* UID of the server */
  gid_t gid;			     /* GID of the server */
  int debug;           /* level of debuging */
  char * root;         /* default working place */
  int banner;          /* file desc of the banner */
  char * redirect;     /* where to send the debuging output */
  int sockfd;          /* socket of port */
  LIMIT * limit;       /* restriction a la wu-ftpd */ 
} SERVER;


/* double link list of clients */
typedef struct CLIENT {
  char * user;          /* user name */
  struct sockaddr_in addr; /* client address */
  char * passwd;        /* user passwd, XXX */
  uid_t uid;			      /* UID of the client */
  gid_t gid;			      /* GID of the client */
  SERVER * server;      /* entry for the server */
  char * cwd;           /* the current working directory */
  char * root;          /* restricted to directory, emulate chroot */
  int auth;             /* authorisation */
#define AUTH_USER    1  /* normal user */
#define AUTH_GUEST   2  /* a guest user */
#define AUTH_ANON    3  /* anonymous user */
  time_t timestamp;     /* when did the connection arrives, for timeout */
  int ostate;           /* The save state we were */
  int state;            /* The state we're in */
# define S_COMM     0   /* command state */
# define S_COMM_W   1   /* send a ack. to cntrl channel */
# define S_DATA     2   /* data state ready */
# define S_DATA_W   3   /* data xfert state sending */
# define S_DATA_R   4   /* data xfert state uploading */
# define S_PROG_CON 5   /* connect() in progress */
# define S_PROG_ACC 6   /* accept() in progress */
# define S_FIN      7   /* end connection, Finish */
  int sock_con;         /* control socket for this connection */
  struct                /* short ack sent to client */
  {
    char * msg;
    off_t size;
    off_t nwritten;
  } reply;
  int sock_dat;         /* data socket for this connection */
  struct
  {
    char * name;
    caddr_t cursor;  /* pointer to mmap file */
    off_t size;
    off_t nwritten;
  } file;
  struct CLIENT * prev;
  struct CLIENT * next;
} CLIENT;

extern char * strerror __P((int));

#ifndef HAVE_SNPRINTF
extern int snprintf __P((char *, size_t, const char *, ...));
extern int vsnprintf __P((char *, size_t, const char *, va_list));
#endif


extern int ftp_cdup __P((char *, CLIENT *));
extern int ftp_cwd __P((char *, CLIENT *));
extern int ftp_list __P((char *, CLIENT *));
extern int ftp_pass __P((char *, CLIENT *));
extern int ftp_pasv __P((char *, CLIENT *));
extern int ftp_port __P((char *, CLIENT *));
extern int ftp_pwd __P((char *, CLIENT *));
extern int ftp_quit __P((char *, CLIENT *));
extern int ftp_retr __P((char *, CLIENT *));
extern int ftp_size __P((char *, CLIENT *));
extern int ftp_syst __P((char *, CLIENT *));
extern int ftp_type __P((char *, CLIENT *));
extern int ftp_user __P((char *, CLIENT *));
extern int not_done __P((char *, CLIENT *));

/* XXX:this should be in config.h or whatever but not here */
#define ANON_FTP 1

#endif  /* _YAFTPD_H */
